@@ -2,9 +2,10 @@ import re
 import json
 import httpx
 import base64
-import urllib.parse
+import asyncio
 from io import IOBase
 from typing import Tuple
+from urllib.parse import urlparse
 
 from .models import RequestModel, Message
 from .utils import (
@@ -27,8 +28,6 @@ from .utils import (
 gemini_max_token_65k_models = ["gemini-2.5-pro", "gemini-2.0-pro", "gemini-2.0-flash-thinking", "gemini-2.5-flash"]
 
 async def get_gemini_payload(request, engine, provider, api_key=None):
-    import re
-
     headers = {
         'Content-Type': 'application/json'
     }
@@ -42,7 +41,7 @@ async def get_gemini_payload(request, engine, provider, api_key=None):
     else:
         gemini_stream = "generateContent"
     url = provider['base_url']
-    parsed_url = urllib.parse.urlparse(url)
+    parsed_url = urlparse(url)
     if "/v1beta" in parsed_url.path:
         api_version = "v1beta"
     else:
@@ -176,6 +175,24 @@ async def get_gemini_payload(request, engine, provider, api_key=None):
     ]
     generation_config = {}
 
+    def process_tool_parameters(data):
+        if isinstance(data, dict):
+            # 移除 Gemini 不支持的 'additionalProperties'
+            data.pop("additionalProperties", None)
+
+            # 将 'default' 值移入 'description'
+            if "default" in data:
+                default_value = data.pop("default")
+                description = data.get("description", "")
+                data["description"] = f"{description}\nDefault: {default_value}"
+
+            # 递归处理
+            for value in data.values():
+                process_tool_parameters(value)
+        elif isinstance(data, list):
+            for item in data:
+                process_tool_parameters(item)
+
     for field, value in request.model_dump(exclude_unset=True).items():
         if field not in miss_fields and value is not None:
             if field == "tools" and "gemini-2.0-flash-thinking" in original_model:
@@ -185,22 +202,9 @@ async def get_gemini_payload(request, engine, provider, api_key=None):
                 processed_tools = []
                 for tool in value:
                     function_def = tool["function"]
-                    # gemini不支持parameters里面的additionalProperties字段，需要删除
-                    if safe_get(function_def, "parameters", "additionalProperties", default=None) is not None:
-                        del function_def["parameters"]["additionalProperties"]
+                    if "parameters" in function_def:
+                        process_tool_parameters(function_def["parameters"])
 
-                    # 处理 parameters.properties 中的 default 字段
-                    if safe_get(function_def, "parameters", "properties", default=None):
-                        for prop_value in function_def["parameters"]["properties"].values():
-                            if "additionalProperties" in prop_value:
-                                del prop_value["additionalProperties"]
-                            if "default" in prop_value:
-                                # 将 default 值添加到 description 中
-                                default_value = prop_value["default"]
-                                description = prop_value.get("description", "")
-                                prop_value["description"] = f"{description}\nDefault: {default_value}"
-                                # 删除 default 字段
-                                del prop_value["default"]
                     if function_def["name"] != "googleSearch" and function_def["name"] != "googleSearch":
                         processed_tools.append({"function": function_def})
 
@@ -333,11 +337,11 @@ def create_jwt(client_email, private_key):
     segments.append(base64.urlsafe_b64encode(signature).rstrip(b'='))
     return b'.'.join(segments).decode()
 
-def get_access_token(client_email, private_key):
-    jwt = create_jwt(client_email, private_key)
+async def get_access_token(client_email, private_key):
+    jwt = await asyncio.to_thread(create_jwt, client_email, private_key)
 
-    with httpx.Client() as client:
-        response = client.post(
+    async with httpx.AsyncClient() as client:
+        response = await client.post(
             "https://oauth2.googleapis.com/token",
             data={
                 "grant_type": "urn:ietf:params:oauth:grant-type:jwt-bearer",
@@ -353,7 +357,7 @@ async def get_vertex_gemini_payload(request, engine, provider, api_key=None):
         'Content-Type': 'application/json'
     }
     if provider.get("client_email") and provider.get("private_key"):
-        access_token = get_access_token(provider['client_email'], provider['private_key'])
+        access_token = await get_access_token(provider['client_email'], provider['private_key'])
         headers['Authorization'] = f"Bearer {access_token}"
     if provider.get("project_id"):
         project_id = provider.get("project_id")
@@ -593,7 +597,7 @@ async def get_vertex_claude_payload(request, engine, provider, api_key=None):
         'Content-Type': 'application/json',
     }
     if provider.get("client_email") and provider.get("private_key"):
-        access_token = get_access_token(provider['client_email'], provider['private_key'])
+        access_token = await get_access_token(provider['client_email'], provider['private_key'])
         headers['Authorization'] = f"Bearer {access_token}"
     if provider.get("project_id"):
         project_id = provider.get("project_id")
@@ -969,7 +973,9 @@ async def get_aws_payload(request, engine, provider, api_key=None):
 
     if provider.get("aws_access_key") and provider.get("aws_secret_key"):
         ACCEPT_HEADER = "application/vnd.amazon.bedrock.payload+json" # 指定接受 Bedrock 流格式
-        amz_date, payload_hash, authorization_header = get_signature(payload, original_model, provider.get("aws_access_key"), provider.get("aws_secret_key"), AWS_REGION, HOST, CONTENT_TYPE, ACCEPT_HEADER)
+        amz_date, payload_hash, authorization_header = await asyncio.to_thread(
+            get_signature, payload, original_model, provider.get("aws_access_key"), provider.get("aws_secret_key"), AWS_REGION, HOST, CONTENT_TYPE, ACCEPT_HEADER
+        )
         headers = {
             'Accept': ACCEPT_HEADER,
             'Content-Type': CONTENT_TYPE,
@@ -1948,9 +1954,9 @@ async def get_embedding_payload(request, engine, provider, api_key=None):
     }
     if api_key:
         headers['Authorization'] = f"Bearer {api_key}"
+
     url = provider['base_url']
     url = BaseAPI(url).embeddings
-
     payload = {
         "input": request.input,
         "model": original_model,
