@@ -6,11 +6,7 @@ import httpx
 import asyncio
 import logging
 import inspect
-import requests
-from typing import Set
-from typing import Union, Optional, Callable, List, Dict, Any
-from pathlib import Path
-
+from typing import Union, Optional, Callable
 
 from .base import BaseLLM
 from ..plugins.registry import registry
@@ -18,27 +14,56 @@ from ..plugins import PLUGINS, get_tools_result_async, function_call_list, updat
 from ..utils.scripts import safe_get, async_generator_to_sync, parse_function_xml, parse_continuous_json, convert_functions_to_xml, remove_xml_tags_and_content
 from ..core.request import prepare_request_payload
 from ..core.response import fetch_response_stream, fetch_response
+from ..architext.architext import Messages, SystemMessage, UserMessage, AssistantMessage, ToolCalls, ToolResults, Texts, RoleMessage, Images, Files
 
-def get_filtered_keys_from_object(obj: object, *keys: str) -> Set[str]:
-    """
-    Get filtered list of object variable names.
-    :param keys: List of keys to include. If the first key is "not", the remaining keys will be removed from the class keys.
-    :return: List of class keys.
-    """
-    class_keys = obj.__dict__.keys()
-    if not keys:
-        return set(class_keys)
+class APITimeoutError(Exception):
+    """Custom exception for API timeout errors."""
+    pass
+
+class ValidationError(Exception):
+    """Custom exception for response validation errors."""
+    def __init__(self, message, response_text):
+        super().__init__(message)
+        self.response_text = response_text
+
+class EmptyResponseError(Exception):
+    """Custom exception for empty API responses."""
+    pass
+
+class ModelNotFoundError(Exception):
+    """Custom exception for model not found (404) errors."""
+    pass
+
+class RateLimitError(Exception):
+    """Custom exception for rate limit (429) errors."""
+    pass
+
+class BadRequestError(Exception):
+    """Custom exception for bad request (400) errors."""
+    pass
+
+class HTTPError(Exception):
+    """Custom exception for HTTP 500 errors."""
+    pass
+
+class InputTokenCountExceededError(Exception):
+    """Custom exception for input token count exceeding the maximum."""
+    pass
+
+class ConfigurationError(Exception):
+    """Custom exception for configuration errors."""
+    pass
+
+class RetryFailedError(Exception):
+    """Custom exception for retry failures."""
+    pass
+
+class TaskComplete(Exception):
+    """Exception-like signal to indicate the task is complete."""
+    def __init__(self, message):
+        self.completion_message = message
+        super().__init__(f"Task completed with message: {message}")
 
-    # Remove the passed keys from the class keys.
-    if keys[0] == "not":
-        return {key for key in class_keys if key not in keys[1:]}
-    # Check if all passed keys are valid
-    if invalid_keys := set(keys) - class_keys:
-        raise ValueError(
-            f"Invalid keys: {invalid_keys}",
-        )
-    # Only return specified keys that are in class_keys
-    return {key for key in keys if key in class_keys}
 
 class chatgpt(BaseLLM):
     """
@@ -64,28 +89,22 @@ class chatgpt(BaseLLM):
         print_log: bool = False,
         tools: Optional[Union[list, str, Callable]] = [],
         function_call_max_loop: int = 3,
-        cut_history_by_function_name: str = "",
         cache_messages: list = None,
         logger: logging.Logger = None,
+        check_done: bool = False,
     ) -> None:
         """
         Initialize Chatbot with API key (from https://platform.openai.com/account/api-keys)
         """
         super().__init__(api_key, engine, api_url, system_prompt, proxy, timeout, max_tokens, temperature, top_p, presence_penalty, frequency_penalty, reply_count, truncate_limit, use_plugins=use_plugins, print_log=print_log)
-        self.conversation: dict[str, list[dict]] = {
-            "default": [
-                {
-                    "role": "system",
-                    "content": self.system_prompt,
-                },
-            ],
+        self.conversation: dict[str, Messages] = {
+            "default": Messages(SystemMessage(self.system_prompt)),
         }
         if cache_messages:
             self.conversation["default"] = cache_messages
         self.function_calls_counter = {}
         self.function_call_max_loop = function_call_max_loop
-        self.cut_history_by_function_name = cut_history_by_function_name
-        self.latest_file_content = {}
+        self.check_done = check_done
 
         if logger:
             self.logger = logger
@@ -138,91 +157,48 @@ class chatgpt(BaseLLM):
         if convo_id not in self.conversation:
             self.reset(convo_id=convo_id)
         if function_name == "" and message:
-            self.conversation[convo_id].append({"role": role, "content": message})
+            self.conversation[convo_id].append(RoleMessage(role, message))
         elif function_name != "" and message:
-            # 删除从 cut_history_by_function_name 以后的所有历史记录
-            if function_name == self.cut_history_by_function_name:
-                matching_message = next(filter(lambda x: safe_get(x, "tool_calls", 0, "function", "name", default="") == 'get_next_pdf', self.conversation[convo_id]), None)
-                if matching_message is not None:
-                    self.conversation[convo_id] = self.conversation[convo_id][:self.conversation[convo_id].index(matching_message)]
-
             if not (all(value == False for value in self.plugins.values()) or self.use_plugins == False):
-                self.conversation[convo_id].append({
-                    "role": "assistant",
-                    "tool_calls": [
-                        {
-                            "id": function_call_id,
-                            "type": "function",
-                            "function": {
-                                "name": function_name,
-                                "arguments": function_arguments,
-                            },
-                        }
-                    ],
-                    })
-                self.conversation[convo_id].append({"role": role, "tool_call_id": function_call_id, "content": message})
+                tool_calls = [
+                    {
+                        "id": function_call_id,
+                        "type": "function",
+                        "function": {
+                            "name": function_name,
+                            "arguments": function_arguments,
+                        },
+                    }
+                ]
+                self.conversation[convo_id].append(ToolCalls(tool_calls))
+                self.conversation[convo_id].append(ToolResults(tool_call_id=function_call_id, content=message))
             else:
                 last_user_message = self.conversation[convo_id][-1]["content"]
                 if last_user_message != message:
-                    image_message_list = []
+                    image_message_list = UserMessage()
                     if isinstance(function_arguments, str):
                         functions_list = json.loads(function_arguments)
                     else:
                         functions_list = function_arguments
                     for tool_info in functions_list:
                         if tool_info.get("base64_image"):
-                            image_message_list.append({"type": "text", "text": safe_get(tool_info, "parameter", "image_path", default="") + " image:"})
-                            image_message_list.append({
-                                "type": "image_url",
-                                "image_url": {
-                                    "url": tool_info["base64_image"],
-                                }
-                            })
-                    self.conversation[convo_id].append({"role": "assistant", "content": convert_functions_to_xml(function_arguments)})
+                            image_message_list.extend([
+                                safe_get(tool_info, "parameter", "image_path", default="") + " image:",
+                                Images(tool_info["base64_image"]),
+                            ])
+                    self.conversation[convo_id].append(AssistantMessage(convert_functions_to_xml(function_arguments)))
                     if image_message_list:
-                        self.conversation[convo_id].append({"role": "user", "content": [{"type": "text", "text": message}] + image_message_list})
+                        self.conversation[convo_id].append(UserMessage(message + image_message_list))
                     else:
-                        self.conversation[convo_id].append({"role": "user", "content": message})
+                        self.conversation[convo_id].append(UserMessage(message))
                 else:
-                    self.conversation[convo_id].append({"role": "assistant", "content": "我已经执行过这个工具了，接下来我需要做什么？"})
-
+                    self.conversation[convo_id].append(AssistantMessage("我已经执行过这个工具了，接下来我需要做什么？"))
         else:
             self.logger.error(f"error: add_to_conversation message is None or empty, role: {role}, function_name: {function_name}, message: {message}")
 
-        conversation_len = len(self.conversation[convo_id]) - 1
-        message_index = 0
         # if self.print_log:
         #     replaced_text = json.loads(re.sub(r';base64,([A-Za-z0-9+/=]+)', ';base64,***', json.dumps(self.conversation[convo_id])))
         #     self.logger.info(json.dumps(replaced_text, indent=4, ensure_ascii=False))
-        while message_index < conversation_len:
-            if self.conversation[convo_id][message_index]["role"] == self.conversation[convo_id][message_index + 1]["role"]:
-                if self.conversation[convo_id][message_index].get("content") and self.conversation[convo_id][message_index + 1].get("content") \
-                and self.conversation[convo_id][message_index].get("content") != self.conversation[convo_id][message_index + 1].get("content"):
-                    if type(self.conversation[convo_id][message_index + 1]["content"]) == str \
-                    and type(self.conversation[convo_id][message_index]["content"]) == list:
-                        self.conversation[convo_id][message_index + 1]["content"] = [{"type": "text", "text": self.conversation[convo_id][message_index + 1]["content"]}]
-                    if type(self.conversation[convo_id][message_index]["content"]) == str \
-                    and type(self.conversation[convo_id][message_index + 1]["content"]) == list:
-                        self.conversation[convo_id][message_index]["content"] = [{"type": "text", "text": self.conversation[convo_id][message_index]["content"]}]
-                    if type(self.conversation[convo_id][message_index]["content"]) == dict \
-                    and type(self.conversation[convo_id][message_index + 1]["content"]) == str:
-                        self.conversation[convo_id][message_index]["content"] = [self.conversation[convo_id][message_index]["content"]]
-                        self.conversation[convo_id][message_index + 1]["content"] = [{"type": "text", "text": self.conversation[convo_id][message_index + 1]["content"]}]
-                    if type(self.conversation[convo_id][message_index]["content"]) == dict \
-                    and type(self.conversation[convo_id][message_index + 1]["content"]) == list:
-                        self.conversation[convo_id][message_index]["content"] = [self.conversation[convo_id][message_index]["content"]]
-                    if type(self.conversation[convo_id][message_index]["content"]) == dict \
-                    and type(self.conversation[convo_id][message_index + 1]["content"]) == dict:
-                        self.conversation[convo_id][message_index]["content"] = [self.conversation[convo_id][message_index]["content"]]
-                        self.conversation[convo_id][message_index + 1]["content"] = [self.conversation[convo_id][message_index + 1]["content"]]
-                    if type(self.conversation[convo_id][message_index]["content"]) == list \
-                    and type(self.conversation[convo_id][message_index + 1]["content"]) == dict:
-                        self.conversation[convo_id][message_index + 1]["content"] = [self.conversation[convo_id][message_index + 1]["content"]]
-                    self.conversation[convo_id][message_index]["content"] += self.conversation[convo_id][message_index + 1]["content"]
-                self.conversation[convo_id].pop(message_index + 1)
-                conversation_len = conversation_len - 1
-            else:
-                message_index = message_index + 1
 
         history_len = len(self.conversation[convo_id])
 
@@ -260,27 +236,6 @@ class chatgpt(BaseLLM):
             else:
                 break
 
-    def get_latest_file_content(self) -> str:
-        """
-        获取最新文件内容
-        """
-        result = ""
-        if self.latest_file_content:
-            for file_path, content in self.latest_file_content.items():
-                result += (
-                    "<file>"
-                    f"<file_path>{file_path}</file_path>"
-                    f"<file_content>{content}</file_content>"
-                    "</file>\n\n"
-                )
-            if result:
-                result = (
-                    "<latest_file_content>"
-                    f"{result}"
-                    "</latest_file_content>"
-                )
-        return result
-
     async def get_post_body(
         self,
         prompt: str,
@@ -291,8 +246,6 @@ class chatgpt(BaseLLM):
         stream: bool = True,
         **kwargs,
     ):
-        self.conversation[convo_id][0] = {"role": "system","content": self.system_prompt + "\n\n" + self.get_latest_file_content()}
-
         # 构造 provider 信息
         provider = {
             "provider": "openai",
@@ -303,13 +256,19 @@ class chatgpt(BaseLLM):
             "image": True
         }
 
+        done_message = self.conversation[convo_id].provider("done")
+        if self.check_done and done_message:
+            done_message.visible = False
+            if self.conversation[convo_id][-1][-1].name == "done":
+                done_message[-1].visible = True
+
         # 构造请求数据
         request_data = {
             "model": model or self.engine,
-            "messages": copy.deepcopy(self.conversation[convo_id]) if pass_history else [
-                {"role": "system","content": self.system_prompt + "\n\n" + self.get_latest_file_content()},
-                {"role": role, "content": prompt}
-            ],
+            "messages": self.conversation[convo_id].render_latest() if pass_history else Messages(
+                SystemMessage(self.system_prompt, self.conversation[convo_id].provider("files")),
+                UserMessage(prompt)
+            ),
             "stream": stream,
             "temperature": kwargs.get("temperature", self.temperature)
         }
@@ -358,6 +317,7 @@ class chatgpt(BaseLLM):
         system_prompt=None,
         pass_history=9999,
         is_async=False,
+        stream: bool = True,
         **kwargs
     ):
         """
@@ -386,6 +346,7 @@ class chatgpt(BaseLLM):
             elif isinstance(line, (dict, list)):
                 if isinstance(line, dict) and safe_get(line, "choices", 0, "message", "content"):
                     full_response = line["choices"][0]["message"]["content"]
+                    total_tokens = safe_get(line, "usage", "total_tokens", default=0)
                     return full_response
                 else:
                     return str(line)
@@ -404,7 +365,7 @@ class chatgpt(BaseLLM):
 
             resp = json.loads(line) if isinstance(line, str) else line
             if "error" in resp:
-                raise Exception(f"{resp}")
+                raise Exception(json.dumps({"type": "api_error", "details": resp}, ensure_ascii=False))
 
             total_tokens = total_tokens or safe_get(resp, "usage", "total_tokens", default=0)
             delta = safe_get(resp, "choices", 0, "delta")
@@ -456,6 +417,9 @@ class chatgpt(BaseLLM):
             for chunk in process_sync():
                 yield chunk
 
+        if not full_response.strip():
+            raise EmptyResponseError("Response is empty")
+
         if self.print_log:
             self.logger.info(f"total_tokens: {total_tokens}")
 
@@ -465,6 +429,12 @@ class chatgpt(BaseLLM):
         missing_required_params = []
 
         if self.use_plugins == True:
+            if self.check_done:
+                # self.logger.info(f"worker Response: {full_response}")
+                if not full_response.strip().endswith('[done]'):
+                    raise ValidationError("Response is not ended with [done]", response_text=full_response)
+                else:
+                    full_response = full_response.strip().rstrip('[done]')
             full_response = full_response.replace("<tool_code>", "").replace("</tool_code>", "")
             function_parameter = parse_function_xml(full_response)
             if function_parameter:
@@ -506,6 +476,9 @@ class chatgpt(BaseLLM):
                 # 删除 task_complete 跟其他工具一起调用的情况，因为 task_complete 必须单独调用
                 if len(function_parameter) > 1:
                     function_parameter = [tool_dict for tool_dict in function_parameter if tool_dict.get("function_name", "") != "task_complete"]
+                    function_parameter = [tool_dict for tool_dict in function_parameter if tool_dict.get("function_name", "") != "get_task_result"]
+                if len(function_parameter) == 1 and function_parameter[0].get("function_name", "") == "task_complete":
+                    raise TaskComplete(safe_get(function_parameter, 0, "parameter", "message", default="The task has been completed."))
 
                 if self.print_log and invalid_tools:
                     self.logger.error(f"invalid_tools: {invalid_tools}")
@@ -526,8 +499,10 @@ class chatgpt(BaseLLM):
         # 处理函数调用
         if need_function_call and self.use_plugins == True:
             if self.print_log:
-                self.logger.info(f"function_parameter: {function_parameter}")
-                self.logger.info(f"function_full_response: {function_full_response}")
+                if function_parameter:
+                    self.logger.info(f"function_parameter: {function_parameter}")
+                else:
+                    self.logger.info(f"function_full_response: {function_full_response}")
 
             function_response = ""
             # 定义处理单个工具调用的辅助函数
@@ -542,17 +517,13 @@ class chatgpt(BaseLLM):
                 tool_response = ""
                 has_args = safe_get(self.function_call_list, tool_name, "parameters", "required", default=False)
                 if self.function_calls_counter[tool_name] <= self.function_call_max_loop and (tool_args != "{}" or not has_args):
-                    function_call_max_tokens = self.truncate_limit - 1000
-                    if function_call_max_tokens <= 0:
-                        function_call_max_tokens = int(self.truncate_limit / 2)
                     if self.print_log:
-                        self.logger.info(f"function_call {tool_name}, max token: {function_call_max_tokens}")
+                        self.logger.info(f"Tool use, calling: {tool_name}")
 
                     # 处理函数调用结果
                     if is_async:
                         async for chunk in get_tools_result_async(
-                            tool_name, tool_args, function_call_max_tokens,
-                            model or self.engine, chatgpt, kwargs.get('api_key', self.api_key),
+                            tool_name, tool_args, model or self.engine, chatgpt, kwargs.get('api_key', self.api_key),
                             kwargs.get('api_url', self.api_url.chat_url), use_plugins=False, model=model or self.engine,
                             add_message=self.add_to_conversation, convo_id=convo_id, language=language
                         ):
@@ -560,8 +531,7 @@ class chatgpt(BaseLLM):
                     else:
                         async def run_async():
                             async for chunk in get_tools_result_async(
-                                tool_name, tool_args, function_call_max_tokens,
-                                model or self.engine, chatgpt, kwargs.get('api_key', self.api_key),
+                                tool_name, tool_args, model or self.engine, chatgpt, kwargs.get('api_key', self.api_key),
                                 kwargs.get('api_url', self.api_url.chat_url), use_plugins=False, model=model or self.engine,
                                 add_message=self.add_to_conversation, convo_id=convo_id, language=language
                             ):
@@ -614,7 +584,7 @@ class chatgpt(BaseLLM):
                         else:
                             yield chunk
                 if tool_name == "read_file" and "<tool_error>" not in tool_response:
-                    self.latest_file_content[tool_info['parameter']["file_path"]] = tool_response
+                    self.conversation[convo_id].provider("files").update(tool_info['parameter']["file_path"], tool_response)
                     all_responses.append(f"[{tool_name}({tool_args}) Result]:\n\nRead file successfully! The file content has been updated in the tag <latest_file_content>.")
                 elif tool_name == "write_to_file" and "<tool_error>" not in tool_response:
                     all_responses.append(f"[{tool_name} Result]:\n\n{tool_response}")
@@ -627,6 +597,9 @@ class chatgpt(BaseLLM):
                 else:
                     all_responses.append(f"[{tool_name}({tool_args}) Result]:\n\n{tool_response}")
 
+            if self.check_done:
+                all_responses.append("Your message **must** end with [done] to signify the end of your output.")
+
             # 合并所有工具响应
             function_response = "\n\n".join(all_responses).strip()
             if missing_required_params:
@@ -647,7 +620,7 @@ class chatgpt(BaseLLM):
                     model=model or self.engine, function_arguments=function_full_response,
                     function_call_id=function_call_id, api_key=kwargs.get('api_key', self.api_key),
                     api_url=kwargs.get('api_url', self.api_url.chat_url),
-                    plugins=kwargs.get("plugins", self.plugins), system_prompt=system_prompt
+                    plugins=kwargs.get("plugins", self.plugins), system_prompt=system_prompt, stream=stream
                 ):
                     yield chunk
             else:
@@ -657,7 +630,7 @@ class chatgpt(BaseLLM):
                     model=model or self.engine, function_arguments=function_full_response,
                     function_call_id=function_call_id, api_key=kwargs.get('api_key', self.api_key),
                     api_url=kwargs.get('api_url', self.api_url.chat_url),
-                    plugins=kwargs.get("plugins", self.plugins), system_prompt=system_prompt
+                    plugins=kwargs.get("plugins", self.plugins), system_prompt=system_prompt, stream=stream
                 ):
                     yield chunk
         else:
@@ -707,13 +680,19 @@ class chatgpt(BaseLLM):
 
         # 打印日志
         if self.print_log:
-            self.logger.info(f"api_url: {kwargs.get('api_url', self.api_url.chat_url)}, {url}")
-            self.logger.info(f"api_key: {kwargs.get('api_key', self.api_key)}")
+            self.logger.debug(f"api_url: {kwargs.get('api_url', self.api_url.chat_url)}")
+            self.logger.debug(f"api_key: {kwargs.get('api_key', self.api_key)}")
+        need_done_prompt = False
 
         # 发送请求并处理响应
-        for i in range(3):
+        retry_times = 0
+        error_to_raise = None
+        while True:
+            tmp_post_json = copy.deepcopy(json_post)
+            if need_done_prompt:
+                tmp_post_json["messages"].extend(need_done_prompt)
             if self.print_log:
-                replaced_text = json.loads(re.sub(r';base64,([A-Za-z0-9+/=]+)', ';base64,***', json.dumps(json_post)))
+                replaced_text = json.loads(re.sub(r';base64,([A-Za-z0-9+/=]+)', ';base64,***', json.dumps(tmp_post_json)))
                 replaced_text_str = json.dumps(replaced_text, indent=4, ensure_ascii=False)
                 self.logger.info(f"Request Body:\n{replaced_text_str}")
 
@@ -739,39 +718,94 @@ class chatgpt(BaseLLM):
                 else:
                     if stream:
                         generator = fetch_response_stream(
-                            self.aclient, url, headers, json_post, engine_type, model or self.engine,
+                            self.aclient, url, headers, tmp_post_json, engine_type, model or self.engine,
                         )
                     else:
                         generator = fetch_response(
-                            self.aclient, url, headers, json_post, engine_type, model or self.engine,
+                            self.aclient, url, headers, tmp_post_json, engine_type, model or self.engine,
                         )
 
                 # 处理正常响应
+                index = 0
                 async for processed_chunk in self._process_stream_response(
                     generator, convo_id=convo_id, function_name=function_name,
                     total_tokens=total_tokens, function_arguments=function_arguments,
                     function_call_id=function_call_id, model=model, language=language,
-                    system_prompt=system_prompt, pass_history=pass_history, is_async=True, **kwargs
+                    system_prompt=system_prompt, pass_history=pass_history, is_async=True, stream=stream, **kwargs
                 ):
+                    if index == 0:
+                        if "HTTP Error', 'status_code': 524" in processed_chunk:
+                            raise APITimeoutError("Response timeout")
+                        if "HTTP Error', 'status_code': 404" in processed_chunk:
+                            raise ModelNotFoundError(f"Model: {model or self.engine} not found!")
+                        if "HTTP Error', 'status_code': 429" in processed_chunk:
+                            raise RateLimitError(f"Rate limit exceeded for model: {model or self.engine}")
+                        if "HTTP Error', 'status_code': 413" in processed_chunk:
+                            raise InputTokenCountExceededError(processed_chunk)
+                        if "HTTP Error', 'status_code': 400" in processed_chunk:
+                            raise BadRequestError(f"Bad Request: {processed_chunk}")
+                        if "HTTP Error', 'status_code': " in processed_chunk:
+                            raise HTTPError(f"HTTP Error: {processed_chunk}")
                     yield processed_chunk
+                    index += 1
 
                 # 成功处理，跳出重试循环
                 break
-            except (httpx.ConnectError, httpx.ReadTimeout):
-                self.logger.error("连接或读取超时错误，请检查服务器状态或网络连接。")
+            except (httpx.ConnectError, httpx.ReadTimeout, httpx.PoolTimeout):
+                self.logger.error("Connection or read timeout.")
                 return # Stop iteration
             except httpx.RemoteProtocolError:
                 continue
+            except httpx.ReadError as e:
+                self.logger.warning(f"{e}, retrying...")
+                continue
+            except APITimeoutError:
+                self.logger.warning("API response timeout (524), retrying...")
+                continue
+            except HTTPError as e:
+                self.logger.warning(f"{e}, retrying...")
+                continue
+            except RateLimitError as e:
+                self.logger.warning(f"{e}, retrying...")
+                continue
+            except InputTokenCountExceededError as e:
+                self.logger.error(f"The request body is too long: {e}")
+                error_to_raise = e
+                break
+            except BadRequestError as e:
+                self.logger.error(f"Bad request error: {e}")
+                raise
+            except GeneratorExit:
+                self.logger.warning("GeneratorExit caught, closing stream.")
+                break
+            except ValidationError as e:
+                self.logger.warning(f"Validation failed: {e}. Retrying with corrective prompt.")
+                need_done_prompt = [
+                    {"role": "assistant", "content": e.response_text},
+                    {"role": "user", "content": "你的消息没有以[done]结尾，请重新输出"}
+                ]
+                continue
+            except EmptyResponseError as e:
+                self.logger.warning(f"{e}, retrying...")
+                continue
+            except TaskComplete as e:
+                raise
+            except ModelNotFoundError as e:
+                raise
             except Exception as e:
-                self.logger.error(f"发生了未预料的错误：{e}")
+                self.logger.error(f"{e}")
                 import traceback
                 self.logger.error(traceback.format_exc())
                 if "Invalid URL" in str(e):
-                    e = "您输入了无效的API URL，请使用正确的URL并使用`/start`命令重新设置API URL。具体错误如下：\n\n" + str(e)
-                    raise Exception(f"{e}")
+                    error_message = "您输入了无效的API URL，请使用正确的URL并使用`/start`命令重新设置API URL。具体错误如下：\n\n" + str(e)
+                    raise ConfigurationError(error_message)
                 # 最后一次重试失败，向上抛出异常
-                if i == 2:
-                    raise Exception(f"{e}")
+                retry_times += 1
+                if retry_times == 9:
+                    raise RetryFailedError(str(e))
+
+        if error_to_raise:
+            raise error_to_raise
 
     def ask_stream(
         self,
@@ -851,7 +885,7 @@ class chatgpt(BaseLLM):
             convo_id=convo_id,
             pass_history=pass_history,
             model=model or self.engine,
-            stream=False,
+            stream=True,
             **kwargs,
         )
         full_response: str = "".join([r async for r in response])
@@ -875,7 +909,7 @@ class chatgpt(BaseLLM):
             convo_id=convo_id,
             pass_history=pass_history,
             model=model or self.engine,
-            stream=False,
+            stream=True,
             **kwargs,
         )
         full_response: str = "".join([r for r in response])
@@ -893,162 +927,8 @@ class chatgpt(BaseLLM):
         Reset the conversation
         """
         self.system_prompt = system_prompt or self.system_prompt
-        self.latest_file_content = {}
-        self.conversation[convo_id] = [
-            {"role": "system", "content": self.system_prompt},
-        ]
+        self.conversation[convo_id] = Messages(
+            SystemMessage(Texts("system_prompt", self.system_prompt), self.conversation[convo_id].provider("files")),
+        )
         self.tokens_usage[convo_id] = 0
         self.current_tokens[convo_id] = 0
-
-    def save(self, file: str, *keys: str) -> None:
-        """
-        Save the Chatbot configuration to a JSON file
-        """
-        with open(file, "w", encoding="utf-8") as f:
-            data = {
-                key: self.__dict__[key]
-                for key in get_filtered_keys_from_object(self, *keys)
-            }
-            # saves session.proxies dict as session
-            # leave this here for compatibility
-            data["session"] = data["proxy"]
-            del data["aclient"]
-            json.dump(
-                data,
-                f,
-                indent=2,
-            )
-
-    def load(self, file: Path, *keys_: str) -> None:
-        """
-        Load the Chatbot configuration from a JSON file
-        """
-        with open(file, encoding="utf-8") as f:
-            # load json, if session is in keys, load proxies
-            loaded_config = json.load(f)
-            keys = get_filtered_keys_from_object(self, *keys_)
-
-            if (
-                "session" in keys
-                and loaded_config["session"]
-                or "proxy" in keys
-                and loaded_config["proxy"]
-            ):
-                self.proxy = loaded_config.get("session", loaded_config["proxy"])
-                self.session = httpx.Client(
-                    follow_redirects=True,
-                    proxies=self.proxy,
-                    timeout=self.timeout,
-                    cookies=self.session.cookies,
-                    headers=self.session.headers,
-                )
-                self.aclient = httpx.AsyncClient(
-                    follow_redirects=True,
-                    proxies=self.proxy,
-                    timeout=self.timeout,
-                    cookies=self.session.cookies,
-                    headers=self.session.headers,
-                )
-            if "session" in keys:
-                keys.remove("session")
-            if "aclient" in keys:
-                keys.remove("aclient")
-            self.__dict__.update({key: loaded_config[key] for key in keys})
-
-    def _handle_response_error_common(self, response_text, json_post):
-        """通用的响应错误处理逻辑，适用于同步和异步场景"""
-        try:
-            # 检查内容审核失败
-            if "Content did not pass the moral check" in response_text:
-                return json_post, False, f"内容未通过道德检查：{response_text[:400]}"
-
-            # 处理函数调用相关错误
-            if "function calling" in response_text:
-                if "tools" in json_post:
-                    del json_post["tools"]
-                if "tool_choice" in json_post:
-                    del json_post["tool_choice"]
-                return json_post, True, None
-
-            # 处理请求格式错误
-            elif "invalid_request_error" in response_text:
-                for index, mess in enumerate(json_post["messages"]):
-                    if type(mess["content"]) == list and "text" in mess["content"][0]:
-                        json_post["messages"][index] = {
-                            "role": mess["role"],
-                            "content": mess["content"][0]["text"]
-                        }
-                return json_post, True, None
-
-            # 处理角色不允许错误
-            elif "'function' is not an allowed role" in response_text:
-                if json_post["messages"][-1]["role"] == "tool":
-                    mess = json_post["messages"][-1]
-                    json_post["messages"][-1] = {
-                        "role": "assistant",
-                        "name": mess["name"],
-                        "content": mess["content"]
-                    }
-                return json_post, True, None
-
-            # 处理服务器繁忙错误
-            elif "Sorry, server is busy" in response_text:
-                for index, mess in enumerate(json_post["messages"]):
-                    if type(mess["content"]) == list and "text" in mess["content"][0]:
-                        json_post["messages"][index] = {
-                            "role": mess["role"],
-                            "content": mess["content"][0]["text"]
-                        }
-                return json_post, True, None
-
-            # 处理token超限错误
-            elif "is not possible because the prompts occupy" in response_text:
-                max_tokens = re.findall(r"only\s(\d+)\stokens", response_text)
-                if max_tokens:
-                    json_post["max_tokens"] = int(max_tokens[0])
-                    return json_post, True, None
-
-            # 默认移除工具相关设置
-            else:
-                if "tools" in json_post:
-                    del json_post["tools"]
-                if "tool_choice" in json_post:
-                    del json_post["tool_choice"]
-                return json_post, True, None
-
-        except Exception as e:
-            self.logger.error(f"处理响应错误时出现异常: {e}")
-            return json_post, False, str(e)
-
-    def _handle_response_error_sync(self, response, json_post):
-        """处理API响应错误并相应地修改请求体（同步版本）"""
-        response_text = response.text
-
-        # 处理空响应
-        if response.status_code == 200 and response_text == "":
-            for index, mess in enumerate(json_post["messages"]):
-                if type(mess["content"]) == list and "text" in mess["content"][0]:
-                    json_post["messages"][index] = {
-                        "role": mess["role"],
-                        "content": mess["content"][0]["text"]
-                    }
-            return json_post, True
-
-        json_post, should_retry, error_msg = self._handle_response_error_common(response_text, json_post)
-
-        if error_msg:
-            raise Exception(f"{response.status_code} {response.reason} {error_msg}")
-
-        return json_post, should_retry
-
-    async def _handle_response_error(self, response, json_post):
-        """处理API响应错误并相应地修改请求体（异步版本）"""
-        await response.aread()
-        response_text = response.text
-
-        json_post, should_retry, error_msg = self._handle_response_error_common(response_text, json_post)
-
-        if error_msg:
-            raise Exception(f"{response.status_code} {response.reason_phrase} {error_msg}")
-
-        return json_post, should_retry
\ No newline at end of file
